<?php
/**
 * TestLink Open Source Project - http://testlink.sourceforge.net/
 *
 * Filename $RCSfile$
 *
 * The interefaces of TestLink - Backlog (BTS).
 * TestLink connect to Backlog via XML-RPC.
 *
 *
 * @version $Revision$
 * @modified $Date$ $Author$
 *
 *
 * @author Tomonari Nakamura
 *
 **/

/** Interface name */
define('BUG_INTERFACE_CLASSNAME', 'backlogInterface');

// This class use XML-RPC.
require_once("XML/RPC.php");
$GLOBALS['XML_RPC_defencoding'] = "UTF-8";

class backlogInterface extends bugtrackingInterface
{
    var $dbHost = null;
    var $enterBugURL = null;
   	var $showBugURL = null;

    var $dbConnection = null;
    var $bConnected = false;

    var $dbCharSet = null;

    // Backlog Variables
    var $xmlrpcClient = null;
    var $currentTestProjectName = null;
    var $currentBtsProjectName = null;

    /**
     * Constructor of bugtrackingInterface
     * put special initialization in here
     *
     * @version 1.0
     * @author Tomonari Nakamura
     **/
    function backlogInterface()
    {
        // Do nothing at constructor
    }

    /**
     * this function establishes the database connection to the
     * bugtracking system
     *
     * @return bool returns true if the db connection was established and the
     * db could be selected, false else
     *
     * @version 1.0
     * @author Tomonari Nakamura
     **/
    function connect()
    {
        $this->bConnected = true;
        return $this->bConnected;
    }

    /**
     * this function simply returns the state of the db connection
     *
     * @return bool returns true if the db connection is established, false else
     *
     * @version 1.0
     * @author Tomonari Nakamura
     **/
    function isConnected()
    {
        return $this->bConnected;
    }

    /**
     * this function closes the db connection (if any)
     *
     * @version 1.0
     * @author Tomonari Nakamura
     **/
    function disconnect()
    {
        $this->bConnected = false;
    }

    /**
     * this to return the URL to the bugtracking page for viewing
     * the bug with the given id.
     *
     * @param int id the bug id
     *
     * @return string returns a complete URL to view the given bug, or false if the bug
     *             wasnt found
     *
     * @version 1.0
     * @author Tomonari Nakamura
     **/
    function buildViewBugURL($id)
    {
    	$this->checkConnectionViaXmarpc();

        $issueUrl = $this->showBugURL . "$id";
        return $issueUrl;
    }

    /**
     * overload this to return the status in a readable form for the bug with the given id
     * This function is not directly called by TestLink
     *
     * @param int id the bug id
     *
     * @return any returns the status (in a readable form) of the given bug, or false
     *             if the bug is not found
     *
     * @version 1.0
     * @author Tomonari Nakamura
     **/
    function getBugStatusString($id)
    {
        if($this->checkConnectionViaXmarpc() == false) {
            return '';
        }

		$xmlrpcResponse = $this->sendMethod($id);
        if (!$xmlrpcResponse->faultCode() && ($xmlrpcResponse->value()->structeach() != NULL)) {
		    $statusString = $xmlrpcResponse->value()->structmem("status")->structmem("name")->scalarval();
		    $statusId = $xmlrpcResponse->value()->structmem("status")->structmem("id")->scalarval();

		    if ($statusId == "4") {
				$statusString = "<del>" . $statusString . "</del>";
			}

		}
        else
        {
            $statusString = "Error: Issue '$id' is not registered in Backlog.";
        }

        return $statusString;
    }

    /*
     *
     * FUNCTIONS CALLED BY TestLink:
     *
     */

    /**
     * default implementation for fetching the bug summary from the
     * bugtracking system
     *
     * @param int id the bug id
     *
     * @return string returns the bug summary (if bug is found), or false
     *
     * @version 1.0
     * @author Tomonari Nakamura
     **/
    function getBugSummaryString($id)
    {
        if($this->checkConnectionViaXmarpc() == false) {
            return '';
        }

        $summary_string = "Error: Issue '$id' is not registered in Backlog.";

        $xmlrpcResponse = $this->sendMethod($id);
        if (!$xmlrpcResponse->faultCode() && ($xmlrpcResponse->value()->structeach() != NULL)) {
		    $summary_string = $xmlrpcResponse->value()->structmem("summary")->scalarval();
        }

        return $summary_string;
     }

	/**
	 * returns the URL which should be displayed for entering bugs
	 *
	 * @return string returns a complete URL
	 *
	 * @version 1.0
	 *
     * @author Tomonari Nakamura
	 **/
	function getEnterBugURL()
	{
		$this->checkConnectionViaXmarpc();

		return $this->enterBugURL;
	}

	/**
	 * default implementation for generating a link to the bugtracking page for viewing
	 * the bug with the given id in a new page
	 *
	 * @param int id the bug id
	 *
	 * @return string returns a complete URL to view the bug (if found in db)
	 *
	 * @version 1.0
	 * @author Tomonari Nakamura
	 **/
	function buildViewBugLink($bugID,$bWithSummary = false)
	{
		global $tlCfg;

		$link = "<a href='" .$this->buildViewBugURL($bugID) . "' target='_blank'>";

		$status = $this->getBugStatusString($bugID);

		if (!is_null($status)) {
			$link .= $status;
		}
		else {
			$link .= $bugID;
		}
		if ($bWithSummary) {
			$summary = $this->getBugSummaryString($bugID);
			if (!is_null($summary)) {
				$link .= " - " . $summary;
			}
		}

		$link .= "</a>";

		return $link;
	}

	 /**
	 * checks a bug id for validity
	 *
	 * @return bool returns true if the bugid has the right format, false else
	 **/
	function checkBugID($id)
	{
		$this->checkConnectionViaXmarpc();
		$status_ok = ereg("^$this->currentBtsProjectName-[0-9]+$", $id);

		return $status_ok;
	}

     /**
     * checks is bug id is present on BTS
     *
     * @param int id the bug id
     *
     * @return if the bug id exest it returns true; otherwise false
     *
     * @version 1.0
     * @author Tomonari Nakamura
     **/
    function checkBugID_existence($id)
    {
        $this->checkConnectionViaXmarpc();
		$xmlrpcResponse = $this->sendMethod($id);
        $statusOk = !$xmlrpcResponse->faultCode() && ($xmlrpcResponse->value()->structeach() != NULL);

        return $statusOk;
    }

    /**
     * Check the connection of XML-RPC each Test Project
     *
     * @param int id the bug id
     *
     * @return if the specified Backlog project exest it returns true; otherwise false
     *
     * @version 1.0
     * @author Tomonari Nakamura
     **/
    function checkConnectionViaXmarpc()
    {
        global $g_interface_bugs_project_name_mapping;

        $tprojectName = isset($_SESSION['testprojectName']) ? $_SESSION['testprojectName'] : 'xx';

        if ($this->currentTestProjectName != $tprojectName) {
            if(!isset($g_interface_bugs_project_name_mapping[$tprojectName])) {
                $this->bConnected = false;
                return false;
            }
            $this->currentTestProjectName = $tprojectName;
            $this->currentBtsProjectName = $g_interface_bugs_project_name_mapping[$tprojectName];
			$this->dbHost = BUG_TRACK_DB_HOST;
			$this->xmlrpcClient = new XML_RPC_client("/XML-RPC", $this->dbHost, 443);
			$this->xmlrpcClient->setCredentials(BUG_TRACK_DB_USER, BUG_TRACK_DB_PASS);
			$this->enterBugURL = BUG_TRACK_ENTER_BUG_HREF . $this->currentBtsProjectName;
			$this->showBugURL = BUG_TRACK_HREF;
        }

        return true;
    }

    /**
     * Send XML-PRC method
     *
     * @param int id the bug id
     *
     * @return XML-RPC response
     *
     * @version 1.0
     * @author Tomonari Nakamura
     **/
    function sendMethod($id) {
        $params = array(new XML_RPC_Value($id, 'string'));
		$message = new XML_RPC_Message('backlog.getIssue', $params);

        return $this->xmlrpcClient->send($message);
    }

}
?>